// File:          InputNetwork.cpp
// Date:          January 2018
// Author:        Sanalitro

#include <algorithm>

#include "BSNetwork.hpp"
#include <fstream>

using namespace std;

// Costruttore di default
BSNetwork::BSNetwork()
{
	neuronsNumber = 90;
	samples = 2000; //samples of a complete simulation of each neuron
	winSize = 500; //size of a temporal window of simulation for each neuron
	deltaT = 0.5; // 1000 ms of simulation
	layer1 = new InputNeuron*[neuronsNumber];
	v1 = new double*[neuronsNumber];
	u1 = new double*[neuronsNumber];
	spikes1 = new double*[neuronsNumber];
	fs1 = new vector<double>;
	layer2 = new InputNeuron*[neuronsNumber];
	v2 = new double*[neuronsNumber];
	u2 = new double*[neuronsNumber];
	spikes2 = new double*[neuronsNumber];
	fs2 = new vector<double>;
	weights = new double[neuronsNumber];
	(*fs1).resize(neuronsNumber);
	(*fs2).resize(neuronsNumber);
	for (int i = 0; i < neuronsNumber; ++i)
	{
		layer1[i] = new InputNeuron(deltaT);
		v1[i] = new double[samples];
		u1[i] = new double[samples];
		spikes1[i] = new double[samples];
		layer2[i] = new InputNeuron(deltaT);
		v2[i] = new double[samples];
		u2[i] = new double[samples];
		spikes2[i] = new double[samples];
		weights[i] = .05 * i + 1;
		(*fs1)[i] = 0;
	}
	for (int i = 0; i < neuronsNumber; ++i) {
		for (int j = 0; j < samples; ++j) {
			spikes1[i][j] = 0;
			spikes2[i][j] = 0;
		}
	}
	//outputNeuron = new OutputNeuron(deltaT);
	vout = new double[samples];
	uout = new double[samples];
	this->reset();
}

// Costruttore con parametri
BSNetwork::BSNetwork(int neuronsNumber, int samples, int winSize, double deltaT)
: neuronsNumber(neuronsNumber), samples(samples), winSize(winSize), deltaT(deltaT)
{
            //Memory pre-allocation
	layer1 = new InputNeuron*[neuronsNumber];
	v1 = new double*[neuronsNumber];
	u1 = new double*[neuronsNumber];
	spikes1 = new double*[neuronsNumber];
	fs1 = new vector<double>;
	layer2 = new InputNeuron*[neuronsNumber];
	v2 = new double*[neuronsNumber];
	u2 = new double*[neuronsNumber];
	spikes2 = new double*[neuronsNumber];
	fs2 = new vector<double>;
	weights = new double[neuronsNumber];
	for (int i = 0; i < neuronsNumber; ++i)
	{
		layer1[i] = new InputNeuron(deltaT);
		v1[i] = new double[samples];
		u1[i] = new double[samples];
		spikes1[i] = new double[samples];
		layer2[i] = new InputNeuron(deltaT);
		v2[i] = new double[samples];
		u2[i] = new double[samples];
		spikes2[i] = new double[samples];
		weights[i] = .05 *i + 1;
	}
	//outputNeuron = new OutputNeuron(deltaT);
	vout = new double[samples];
	uout = new double[samples];
	this->reset();
}

// Distruttore
BSNetwork::~BSNetwork()
{
	for (int i = 0; i < neuronsNumber-1; i++)
	{
		delete layer1[i];
		//delete &(v1[i]);
		//delete &(u1[i]);
		//delete &(spikes1[i]);
		delete layer2[i];
		//delete &(v2[i]);
		//delete &(u2[i]);
		//delete &(spikes2[i]);
	}
	delete layer1;
	delete v1;
	delete u1;
	delete spikes1;
	delete fs1;
	delete layer2;
	delete v2;
	delete u2;
	delete spikes2;
	delete fs2;
	//delete outputNeuron;
	//delete vout;
	//delete uout;
	//delete weights;
}

bool BSNetwork::simulate(double firstAngle, double secondAngle, OutputNeuron *outputNeuron)
{
	double firstInput = -fabs(sin(firstAngle)); // inhibitory sin(i)
	double secondInput = fabs(sin(secondAngle - firstAngle)); // excitatory sin(j-i)
	double i_input1, i_input2, i_inputo;
	double fs_th = 1; // spiking frequency saturation due to synapse attenuation
	int n_spikes = 0;
	// int spikesout = 0;

	for (int i = 0; i <  samples; i++)
	{
		// first layer
		for (int j = 0; j < neuronsNumber; j++)
		{
			i_input1 = firstInput + weights[j] * secondInput;
			spikes1[j][i + 1] = spikes1[j][i] + layer1[j]->simulate(i_input1, v1[j][i], u1[j][i], &v1[j][i + 1], &u1[j][i + 1]);
			// each winSize samples we compute the average spiking frequency
			if (i % (winSize-1) == 0 && i!=0)
			{
				n_spikes = spikes1[j][i] - spikes1[j][i - winSize + 1];
				if (n_spikes >= 5)
				{
					(*fs1)[j] = 100 * n_spikes / (winSize * deltaT);
					if ((*fs1)[j] > fs_th)
						(*fs1)[j] = fs_th;
				}
			}
		}
		// second layer
		for (int j = 0; j < neuronsNumber; j++)
		{
			// each winSize samples we compute the average spiking frequency
			if (i % (winSize-1) == 0 && i != 0)
			{
				n_spikes = spikes2[j][i] - spikes2[j][i - winSize + 1];
				if (n_spikes >= 5)
				{
					(*fs2)[j] = 100 * n_spikes / (winSize * deltaT);
					if ((*fs2)[j] > fs_th)
						(*fs2)[j] = fs_th;
				}
			}
			i_input2 = 2 * (*fs1)[j];
			if (j != 0)
				i_input2 += - (*fs1)[j-1] - (*fs2)[j-1];
			if (j != neuronsNumber-1)
				i_input2 += - (*fs1)[j+1] - (*fs2)[j+1];
			else  //ultimo neurone
				i_input2 += - 2 * (*fs1)[j - 1];
			spikes2[j][i + 1] = spikes2[j][i] + layer2[j]->simulate(i_input2, v2[j][i], u2[j][i], &v2[j][i + 1], &u2[j][i + 1]);
		}

		// output neuron
		/* 
		 * Per motivi di performance è stato implementato un workaround
		 * che permetta alla funzione di ritornare prima del termine
		 * della simulazione, nel caso in cui il neurone di output abbia
		 * emesso uno spike. In tal caso, infatti, vuol dire che questo
		 * ha ricevuto un input sufficiente alla sua attivazione e non
		 * avrebbe senso proseguire con la simulazione.
		 * In caso dovesse risultare significativo valutare anche il
		 * numero di spike emessi dal neurone di output, si può
		 * ripristinare il modello sinaptico completo e salvare in una
		 * variabile spikesout il numero di spike
		*/

		i_inputo = 0;

		/*
		for (j = 1; j < neuronsNumber; j++)
			i_inputo += (neuronsNumber - j + 1) * (*fs2)[j] / 10;
		spikesout += outputNeuron->simulate(i_inputo, vo[i], uo[i], &vo[i + 1], &uo[i + 1];
		*/

		vector<double>::const_iterator indw = max_element(fs2->begin(), fs2->end()); // index of the winner neuron of layer 2
		if (*indw > 0) {
			idWinnerNeuron = ((indw+1) - fs2->begin());
			i_inputo = (neuronsNumber - ((indw + 1) - fs2->begin()) + 1) * (*indw) / 10;
		}
		if ((outputNeuron->simulate(i_inputo, vout[i], uout[i], &vout[i + 1], &uout[i + 1])) > 0) {
			cout << "simulate - Vth:" << outputNeuron->getVth() << endl;
			return 1;
		}
			
	}
	return 0;
}

void BSNetwork::update(bool isTaken, bool isFiring)
{
	outputNeuron->update(isTaken, isFiring);
}

void BSNetwork::reset()
{
	for (int i = 0; i < neuronsNumber; i++)
	{
		v1[i][0] = -70;
		u1[i][0] = 0;
		v2[i][0] = -70;
		u2[i][0] = 0;
	}
	vout[0] = -70;
	uout[0] = 0;
}
