// File:          InputNetwork.hpp
// Date:          January 2018
// Author:        Sanalitro

#ifndef BSNETWORK_HPP
#define BSNETWORK_HPP

#include <vector>

#include "InputNeuron.hpp"
#include "OutputNeuron.hpp"

/*
 * Classe che implementa il modello della rete per il Body Size Learning. La
 * rete presenta due input layer costituiti ciascuno da un vettore di 90 neuroni
 * (valore di default) e un output layer costituito da un unico neurone che
 * presenta plasticità sinaptica. La rete può essere simulata per un dato numero
 * di campioni, sia per la fase di learning che per quella di test.
 * La classe mantiene tutte le quantità della rete in forma matriciale.
 */
class  BSNetwork
{

private:
	int neuronsNumber;
	int samples, winSize;
	double deltaT;
	InputNeuron** layer1;
	double** v1;
	double** u1;
	double** spikes1;
	std::vector<double>* fs1;
	InputNeuron** layer2;
	double** v2;
	double** u2;
	double** spikes2;
	std::vector<double>* fs2;
	OutputNeuron* outputNeuron;
	double* vout;
	double* uout;
	double* weights;

public:
	int idWinnerNeuron;
	BSNetwork();
	BSNetwork(int neuronsNumber, int samples, int winSize, double deltaT);
	virtual ~BSNetwork();

	bool simulate(double firstAngle, double secondAngle, OutputNeuron *outputNeuron); // returns the number of spikes emitted by the outputNeuron
	void update(bool isTaken, bool isFiring);
	void reset();

};

#endif
