// File:          Neuron.cpp
// Date:          January 2018
// Author:        Sanalitro

#include "InputNeuron.hpp"

// Constructor
InputNeuron::InputNeuron(double deltaT)
: deltaT(deltaT)
{
	a = 0.2;
	b = 0.26;
	c = -65; // [mV]
	d = 0;
	Ibias = .235; // [nA]
}

// Destructor
InputNeuron::~InputNeuron()
{
}

/*
	This method simulates one sample so as to allow a parallel computation of the
	network (all the neurons, for each iteration of the network, so for each sample, 
	are simulated.)
 */
int InputNeuron::simulate(double sinapticInput, double v0, double u0, double *v, double *u)
{
	double I = Ibias + sinapticInput; // PSP

	*v = v0 + (0.04 * v0 * v0 + 5 * v0 + 140 - u0 + I) * deltaT;
	*u = u0 + (a * (b * v0 - u0)) * deltaT;

	if (*v >= 30.0) {
		*v = c;
		*u = *u + d;
		return 1;
	}
	else return 0;
}
