// File:          OutputNeuron.cpp
// Date:          January 2018
// Author:        Sanalitro

#include "OutputNeuron.hpp"

// Costruttore
OutputNeuron::OutputNeuron(double deltaT)
: deltaT(deltaT)
{
	//Class 1 Izhikevich
	a = 0.02;
	b = -0.1;
	c = -55; // [mV]
	d = 6;
	Ibias = 18; // [nA]
	gA = 1; // activation conductance [kS]
	Vth = 0;
	deltaVth = 0.3;
}

OutputNeuron::OutputNeuron(double deltaT, double vth)
: OutputNeuron(deltaT)
{
	this->Vth = vth;
}

OutputNeuron::OutputNeuron(double deltaT, double vth, double deltaVth)
: OutputNeuron(deltaT)
{
	this->Vth = vth;
	this->deltaVth = deltaVth;
}

//Destructor
OutputNeuron::~OutputNeuron()
{
}

double OutputNeuron::getVth(){ return Vth; }

// Simula un solo campione
int OutputNeuron::simulate(double sinapticInput, double v0, double u0, double *v, double *u)
{
	double I = Ibias + sinapticInput - gA * Vth; // PSP
	
	*v = v0 + (0.04 * v0 * v0 + 5 * v0 + 140 - u0 + I) * deltaT;
	*u = u0 + (a * (b * v0 - u0)) * deltaT;

	if (*v >= 30.0) {
		*v = c;
		*u = *u + d;
		return 1;
	}
	else return 0;
}


double OutputNeuron::update(bool isTaken, bool isFiring)
{
	if (isFiring && !isTaken) // uncorrect attempt
		Vth += deltaVth;
	else{
		if(!isFiring && isTaken) // correct attempt
			{Vth -= deltaVth;}
        	}
			
	return Vth;
}
